require "points"

font = love.graphics.newFont(22)
love.graphics.setFont(font)

animation = {}
playFrame = 1
lastFrameUpdateTime = love.timer.getTime()

--time between frames (in seconds)
DeltaTime = 0.0166
--duration of render (in seconds)
duration = 20
--frames
frame = 1
maxFrames = duration/DeltaTime
--size
renderWidth = love.graphics.getWidth()
renderHeight = love.graphics.getHeight()

seed = os.time()

progress = 0
barLength = love.graphics.getWidth()/1.5
barHeight = love.graphics.getHeight()/5

scrub = {x = 10, y = love.graphics.getHeight() - 22, w = love.graphics.getWidth() - 20, h = 16, held = false}


local canvas = love.graphics.newCanvas(renderWidth, renderHeight)

rendering = true
play_pressed = false
playing = false
screen = "render"

math.randomseed(os.time())
love.load = function()
	points.load(100)
end

--eta counters
eta = 0
etaUpdateTime = 1
startTime = love.timer.getTime()
lastUpdateTime = love.timer.getTime()
timePassed = 0

love.update = function(dt)
	if rendering then
		canvas:renderTo(function()
			render()
		end)
		data = canvas:getImageData()
		data:encode("render_"..frame..".png")
		
		timePassed = love.timer.getTime() - lastUpdateTime
		if timePassed > etaUpdateTime then
			lastUpdateTime = love.timer.getTime()
			framesLeft = maxFrames - frame
			timePassedFull = love.timer.getTime() - startTime
			eta = math.floor((timePassedFull * framesLeft)/frame + 0.5)
		end
		
		progress = frame / maxFrames
		frame = frame + 1
		
		if frame > maxFrames then
			rendering = false
		end
	end
	if screen == "play" then
		if love.timer.getTime() - lastFrameUpdateTime >= DeltaTime then
			lastFrameUpdateTime = love.timer.getTime()
			playFrame = playFrame + 1
			if playFrame > maxFrames then
				playFrame = 1
			end
		end
		if love.mouse.isDown("l") then
			x = love.mouse.getX()
			y = love.mouse.getY()
			if scrub.held then
				playFrame = math.min(math.max(math.floor((x - scrub.x) / scrub.w * (maxFrames - 1) + 0.5) + 1, 1), maxFrames)
			end
		end
	end
end

render = function()
	canvas:clear()
	
	--draw whatever
	points.render(frame/maxFrames, renderWidth, renderHeight)
	
	animation[frame] = love.graphics.newImage(canvas:getImageData())
	
	--update whatever
end

love.mousepressed = function(x, y, b)
	if screen == "render" then
		if x > love.graphics.getWidth()/2 - 120 and
		x < love.graphics.getWidth()/2 - 120 + 240 and
		y > love.graphics.getHeight()/2 + barHeight/2 + 20 and
		y < love.graphics.getHeight()/2 + barHeight/2 + 20 + 50 then
			play_pressed = true
		end
	elseif screen == "play" then
		if x >= scrub.x and x <= scrub.x + scrub.w and
		y >= scrub.y and y <= scrub.y + scrub.h then
			scrub.held = true
		end
	end
end

love.mousereleased = function(x, y, b)
	if screen == "render" then
		if x > love.graphics.getWidth()/2 - 120 and
		x < love.graphics.getWidth()/2 - 120 + 240 and
		y > love.graphics.getHeight()/2 + barHeight/2 + 20 and
		y < love.graphics.getHeight()/2 + barHeight/2 + 20 + 50 then
			playing = true
			screen = "play"
			lastFrameUpdateTime = love.timer.getTime()
		else
			play_pressed = false
		end
	elseif screen == "play" then
		scrub.held = false
	end
end

love.draw = function()
	if screen == "render" then
		love.graphics.setBackgroundColor(50, 50, 50)
		love.graphics.setColor(0, 200, 0)
		love.graphics.rectangle("fill", love.graphics.getWidth()/2 - barLength/2, love.graphics.getHeight()/2 - barHeight/2, barLength*progress, barHeight)
		love.graphics.setColor(0, 0, 0)
		love.graphics.setLineWidth(4)
		love.graphics.rectangle("line", love.graphics.getWidth()/2 - barLength/2, love.graphics.getHeight()/2 - barHeight/2, barLength, barHeight)
		if rendering then
			love.graphics.setColor(100, 100, 100)
			love.graphics.print("RENDERING...", love.graphics.getWidth()/2 - font:getWidth("RENDERING...")/2, love.graphics.getHeight()/2 - barHeight/2 - 40)
			love.graphics.setColor(180, 180, 200)
			love.graphics.rectangle("fill", love.graphics.getWidth()/2 - 120, love.graphics.getHeight()/2 + barHeight/2 + 20, 240, 50)
			love.graphics.setColor(0, 0, 0)
			love.graphics.print("ETA: "..eta.." seconds", love.graphics.getWidth()/2 - font:getWidth("ETA: "..eta.." seconds")/2, love.graphics.getHeight()/2 + barHeight/2 + 32)
		else
			love.graphics.setColor(100, 100, 100)
			love.graphics.print("FINISHED", love.graphics.getWidth()/2 - font:getWidth("FINISHED")/2, love.graphics.getHeight()/2 - barHeight/2 - 40)
			love.graphics.setColor(0, 150, 200)
			love.graphics.rectangle("fill", love.graphics.getWidth()/2 - 120, love.graphics.getHeight()/2 + barHeight/2 + 20, 240, 50)
			love.graphics.setColor(0, 0, 0)
			love.graphics.print("PLAY ANIMATION", love.graphics.getWidth()/2 - font:getWidth("PLAY ANIMATION")/2, love.graphics.getHeight()/2 + barHeight/2 + 32)
		end
	elseif screen == "play" then
		love.graphics.setBackgroundColor(255, 255, 255)
		love.graphics.setColor(255, 255, 255)
		
		love.graphics.draw(animation[playFrame], love.graphics.getWidth()/2 - animation[playFrame]:getWidth()/2, love.graphics.getHeight()/2 - animation[playFrame]:getHeight()/2)
		
		--scale to fit
		--love.graphics.draw(animation[playFrame], 0, 0, 0, love.graphics.getWidth()/renderWidth, love.graphics.getHeight()/renderHeight)
		
		love.graphics.setColor(200, 200, 255, 150)
		love.graphics.rectangle("fill", scrub.x, scrub.y, scrub.w, scrub.h)
		love.graphics.setColor(0, 255, 0, 150)
		love.graphics.rectangle("fill", scrub.x, scrub.y, scrub.w * (playFrame/maxFrames), scrub.h)
		love.graphics.setColor(0, 0, 0, 150)
		love.graphics.setLineWidth(1.5)
		love.graphics.rectangle("line", scrub.x, scrub.y, scrub.w, scrub.h)
		love.graphics.line(scrub.w * (playFrame/maxFrames) + scrub.x, scrub.y,
							scrub.w * (playFrame/maxFrames) + scrub.x, scrub.y + scrub.h)
	end
end